import 'package:flutter_mylibrary/base/request_info.dart';
import 'package:flutter_mylibrary/http/easy_dio.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import '../base_view_state.dart';
import 'base_view_state_getx_controller.dart';


abstract class GetxRequestListController<T> extends BaseViewStateGetxController with BaseViewState{


  /// 页面数据
  List<T> list = [];

  onCompleted(List<T> data) {}


  void initData({bool retry = false}) async{
    if (retry) {
      setBusy();
    }

    List<T> list = await getData();

    onCompleted(list);

    if(!isError){
      if (isEmpty(list)) {
        setError(
          errorType: ViewStateErrorType.noDataError
        );
      }else{
        setIdle();
      }
      refreshController.finishRefresh(success: true);
      refreshController.resetRefreshState();
    }
  }



  ///获取数据
  Future<List<T>> getData() async{

    getOtherData();

    RequestInfo requestInfo = getRequestInfo();


    return EasyDio().request(
        requestInfo?.method,
        requestInfo?.url,
        data: requestInfo?.data,
        publicMethod: requestInfo?.publicMethod,
    ).then((res){
      try{
        if (isNotEmpty(res.data)) {
          list = res.data is List? List<T>.from(res.data.map((e) => requestInfo?.handleRes(e))).toList() : requestInfo?.handleRes(res.data);
        }else{
          errorType = ViewStateErrorType.noDataError;
          setError(
              errorType: errorType,
          );
        }
        return list;
      }catch(e){
        onError(e);

        errorType = ViewStateErrorType.jsonParsingError;

        setError(
            errorType: errorType,
            e: e
        );

        return null;
      }
    }, onError: (e){

      setError(
        errorType: errorType,
        e: e
      );
    });
  }
}