import 'package:flutter_mylibrary/base/base_view_state.dart';
import 'package:flutter_mylibrary/http/easy_dio.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/utils/utils.dart';

import '../../request_info.dart';
import '../view_state_model.dart';

abstract class ViewStateRequestListModel<T> extends ViewStateModel{

  /// 页面数据
  List<T> list = [];


  onCompleted(List<T> data) {}

  void initData({bool retry = false}) async{
    if (retry) {
      setBusy();
    }
    List<T> list = await getData();

    onCompleted(list);

    if(!isError){
      if (isEmpty(list)) {
        setError(
          errorType: ViewStateErrorType.noDataError
        );
      }else{
        setIdle();
      }
      refreshController.finishRefresh(success: true);
      refreshController.resetRefreshState();
    }

  }


  ///获取数据
  Future<List<T>> getData() async{

    getOtherData();

    RequestInfo requestInfo = getRequestInfo();


    return EasyDio().request(
        requestInfo?.method,
        requestInfo?.url,
        data: requestInfo?.data,
        publicMethod: false,
        onError: isInit? null : (e){
          handleError(e);
        }
    ).then((res){
      try{
        if (isNotEmpty(res.data)) {
          list = res.data is List? List<T>.from(res.data.map((e) => requestInfo?.handleRes(e))).toList() : requestInfo?.handleRes(res.data);
        }
        return list;
      }catch(e){
        handleError(e);
        return null;
      }
    });
  }


  void handleError(e){
    refreshController.finishRefresh(success: false);
    setError(
      e: e
    );
  }
}