import 'package:flutter/cupertino.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:flutter_mylibrary/base/base_view_state.dart';
import 'package:flutter_mylibrary/base/request_info.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/http/easy_dio.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';
import '../view_state_model.dart';

abstract class ViewStateRequestModel<T> extends ViewStateModel{

  T data;


  ///获取主数据
  Future<T> initData({bool retry = false, bool refresh = false}) async{
    RequestInfo requestInfo = getRequestInfo();

    if (retry) {
      refreshController = EasyRefreshController();
      setBusy();
    }

    getOtherData();

    await EasyDio().request(
      requestInfo?.method,
      requestInfo?.url,
      data: requestInfo?.data,
      publicMethod: false,
      onFinish: (){
        refreshController.finishRefresh();
      },
      onError: isInit? null : (e){
        if (e is HttpError) {
          setError(e: e);
        }else{
          setError(e: e);
        }
      }
    ).then((res){
      try{
        if (isNotEmpty(res.data)) {
          data = requestInfo?.handleRes(res.data);
          onCompleted(data);
        }
        if (isEmpty(data)) {
          setError(
            errorType: ViewStateErrorType.noDataError
          );
        }
      }catch(e){
          setError(
            e: e
          );
        }
      });

      if (!isError || refresh) {
        setIdle();
      }
      if (isIdle && isEmpty(data)) {
        ///请求无异常但是数据为空需要设置为empty
        errorType = ViewStateErrorType.noDataError;
        setError(
          errorType: errorType
        );
      }
      return data;
  }


  onCompleted(T data) {}


  ///获取一个公共的刷新控件
  Widget getEasyRefreshWidget({Widget emptyWidget, ViewStateModelWidgetBuilder<T> builder, Widget busyWidget, bool showErrorWidget = true, bool showEmptyWidget = true, bool showBusyWidget = true, OnLoadCallback onLoad}){
    Widget emptyErrorWidget = buildRefreshNoDataWidget(viewStateModel: this, busyWidget: busyWidget, emptyDataWidget: emptyWidget, showErrorWidget: showErrorWidget, showEmptyWidget: showEmptyWidget, showBusyWidget: showBusyWidget);
    return EasyRefresh(
      emptyWidget: emptyErrorWidget,
      child: isEmpty(emptyErrorWidget)? builder(data) : Container(),
      onRefresh: isBusy? null : () async{
         await initData(refresh: true);
      },
      onLoad: onLoad,
      controller: refreshController,
    );
  }

}

